import React, { createContext, useContext } from 'react';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { useCancelOrderForm } from '../forms/useCancelOrderForm';

function useExchangeCancelOrderState(props: Parameters<typeof useCancelOrderForm>[0]) {
  const form = useCancelOrderForm(props);
  return form;
}

export type ExchangeCancelOrderContextType = ReturnType<typeof useExchangeCancelOrderState>;

const ExchangeCancelOrderContext = createContext<ExchangeCancelOrderContextType>(null as any);

export function useExchangeCancelOrder() {
  const state = useContext(ExchangeCancelOrderContext);
  if (!state) throw new Error('ExchangeCancelOrderContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading add drops form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function ExchangeCancelOrderProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  ...props
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
} & Parameters<typeof useExchangeCancelOrderState>[0]) {
  const state = useExchangeCancelOrderState(props);

  // TODO: in future this provider might load order for display
  if (false) return renderLoading();
  if (false) return renderError('Error');
  return (
    <ExchangeCancelOrderContext.Provider value={state}>
      {children}
    </ExchangeCancelOrderContext.Provider>
  );
}
