import React, { createContext, useContext, useMemo } from 'react';
import { useTransferAssetForm } from '@onomy/react-hub';
import { FungibleAssetOnChain } from '@onomy/frontend-relay-types';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useTransferAssetState(asset: FungibleAssetOnChain | null) {
  const form = useTransferAssetForm(asset);
  return form;
}

export type TransferAssetContextType = ReturnType<typeof useTransferAssetState>;

const TransferAssetContext = createContext<TransferAssetContextType>(null as any);

export function useTransferAsset() {
  const state = useContext(TransferAssetContext);
  if (!state) throw new Error('TransferAssetContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_LOADING_RENDER_FN() {
  return <PlaceholderLoadingIndicator text="Loading transfer form" />;
}
// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function TransferAssetProvider({
  children,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  asset = null,
}: {
  children: JSX.Element | JSX.Element[];
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
  asset: FungibleAssetOnChain | null;
}) {
  const state = useTransferAssetState(asset);

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return <TransferAssetContext.Provider value={state}>{children}</TransferAssetContext.Provider>;
}
